#! /bin/dash
#
# interactive plotting script

prog=$(basename "$0")

case "$prog" in
  p1|p3) Xmin="Xmin"; Xmax="Xmax";;
     p2) Xmin="Tmin"; Xmax="Tmax";;
esac

xmin="$1"
xmax="$2"
zoom="$3"
shift="$4"
xlabel="$5"
ylabel="$6"
title="$7"
fname="$8"
xstart="$9"
 
# check/set default xmin, xmax, zoom, shift, xlabel, ylabel values
#
if [ -z "$xmin" ]; then
  xmin="0.0"
fi
#
if [ -z "$xmax" ]; then
  xmax="1.0"
fi
#
if [ -z "$zoom" ]; then
  zoom="1.5"
fi
#
if [ -z "$shift" ]; then
  shift="0.5"
fi
#
if [ -z "$xlabel" ]; then
  xlabel="x"
fi
#
if [ -z "$ylabel" ]; then
  ylabel="y"
fi
#
if [ -z "$fname" ]; then
  fname="$prog"
fi
#
if [ -z "$xstart" ]; then
  xstart="$xmax"
fi

# set plot gif file, check/remove old files
#
gif="$$.gif"
#
if [ -s "rm.list" ]; then
  rm -f `cat rm.list`
fi
#
echo "$gif" > rm.list

# get/check option
#
read option
#
nflag=""
save=""
Save="Save"
#
if [ -z "$option" ]; then
  xaxis="checked"
  xopt=""
  yaxis="checked"
  yopt=""
  lines="checked"
  points="checked"
else
  xaxis=""
  xopt="-x"
  yaxis=""
  yopt="-y"
  lines=""
  points=""
  #
  oldxmin="$xmin"
  oldxmax="$xmax"
  #
  case "$option" in
    *Shift\ Left*)
	xmin=$(echo "$oldxmin" "$shift" | awk '{ print $1 - $2; }')
	xmax=$(echo "$oldxmax" "$shift" | awk '{ print $1 - $2; }')
	;;
    *Shift\ Right*)
	xmin=$(echo "$oldxmin" "$shift" | awk '{ print $1 + $2; }')
	xmax=$(echo "$oldxmax" "$shift" | awk '{ print $1 + $2; }')
	;;
    *Zoom\ in*)
	xmin=$(echo "$oldxmin" "$oldxmax" "$zoom" | awk '{ print ($2+$1-($2-$1)/$3)/2; }')
	xmax=$(echo "$oldxmin" "$oldxmax" "$zoom" | awk '{ print ($2+$1+($2-$1)/$3)/2; }')
	;;
    *Zoom\ out*)
	xmin=$(echo "$oldxmin" "$oldxmax" "$zoom" | awk '{ print ($2+$1-($2-$1)*$3)/2; }')
	xmax=$(echo "$oldxmin" "$oldxmax" "$zoom" | awk '{ print ($2+$1+($2-$1)*$3)/2; }')
	;;
    *Newton*)
	nflag="-n $xstart"
	Save="NSave"
	;;
    *NSave*)
	nflag="-n $xstart"
	Save="NSave"
	save="1"
	gif="$fname.gif"
	;;
    *Save*)
	save="1"
	gif="$fname.gif"
	;;
  esac
  #
  case "$option" in
    *X-axis*)
	xaxis="checked"
	xopt=""
	;;
  esac
  #
  case "$option" in
    *Y-axis*)
	yaxis="checked"
	yopt=""
	;;
  esac
  #
  case "$option" in
    *Lines*)
	lines="checked"
	;;
  esac
  #
  case "$option" in
    *Points*)
	points="checked"
	;;
  esac
fi

# set linespoints
#
if [ "$lines" = "checked" ]; then
  if [ "$points" = "checked" ]; then
    linespoints="with linespoints"
  else
    linespoints="with lines"
  fi
elif [ "$points" = "checked" ]; then
  linespoints="with points"
else
  linespoints=""
fi

# set title and labels
#
labels="set title \"$title\"\nset xlabel \"$xlabel\"\nset ylabel \"$ylabel\""

# create plot
#
exec 2>plot.stderr
#
./$prog.exe $nflag $xopt $yopt "$xmin" "$xmax" |
(echo "set term gif\nset output\n $labels\n plot '-' notitle $linespoints"; cat) |
 /usr/bin/gnuplot > "$gif"

# create web page
#
echo "<html><head><title>iPlot/$prog</title></head>
<body bgcolor=white text=black>
<form method=POST action=\"/prog/${COURSE}?iPlot+$prog.c\">
${Xmin}: <input name=args size=10 value=\"$xmin\">
<input type=hidden name=args value=\" \">
${Xmax}: <input name=args size=10 value=\"$xmax\">
<input type=hidden name=args value=\" \">
Zoom: <input name=args size=5 value=\"$zoom\">
<input type=hidden name=args value=\" \">
Shift: <input name=args size=5 value=\"$shift\">
<input type=hidden name=args value=\" '\">
X-label: <input name=args size=15 value=\"$xlabel\">
<input type=hidden name=args value=\"' '\">
Y-label: <input name=args size=15 value=\"$ylabel\">
<input type=hidden name=args value=\"' '\">
<br><input type=submit name=data value=\"Go\">
<input type=submit name=data value=\"Shift Left\">
<input type=submit name=data value=\"Zoom in\">
<input type=submit name=data value=\"Zoom out\">
<input type=submit name=data value=\"Shift Right\">
<input type=hidden name=data value=\" \">
X-axis<input type=checkbox name=data $xaxis value=\"X-axis\">
<input type=hidden name=data value=\" \">
Y-axis<input type=checkbox name=data $yaxis value=\"Y-axis\">
<input type=hidden name=data value=\" \">
Lines<input type=checkbox name=data $lines value=\"Lines\">
<input type=hidden name=data value=\" \">
Points<input type=checkbox name=data $points value=\"Points\">
<input type=hidden name=data value=\" \">
Title: <input name=args size=15 value=\"$title\">
<input type=hidden name=args value=\"'\">
<br><input type=submit name=data value=\"$Save\">
<input type=hidden name=args value=\" '\">
<input name=args size=10 value=\"$fname\">.html
<input type=hidden name=args value=\"'\">
"

if [ "$prog" = "p3" ]; then
  echo "
  <input type=hidden name=data value=\" \">
  <input type=submit name=data value=\"Newton\">
  <input type=hidden name=args value=\" \">
  Xstart: <input name=args size=10 value=\"$xstart\">
  "
fi

# img element for the gif/plot
#
img="<br><img src=\"/prog/${COURSE}?view+iPlot+$gif\" width=640 height=480 alt=\"\">"

# DEBUG:
#
# echo "<pre>
# args = $@
# data = $option
# img = $img
# </pre>"
 
# function to display newton.out file
#
check_newton()
{
  if [ "$prog" = "p3" -a "$nflag" ]; then
    echo "<pre>"
    cat newton.out
    echo "</pre>"
  fi
}

# function to display $prog.c file
#
display_prog()
{
  echo "<br>$prog.c:<pre>"
  sed -e 's/\&/\&amp;/g' -e 's/</\&lt;/g' -e 's/>/\&gt;/g' "$prog.c"
  echo "</pre>"
}

# check Save option
#
if [ -n "$save" ]; then
  echo "<html><head><title>iPlot/$prog/$fname</title></head>
  <body bgcolor=white text=black>"			 > "$fname.html"
  echo "$img"						>> "$fname.html"
  check_newton						>> "$fname.html"
  display_prog						>> "$fname.html"
  echo "</body></html>"					>> "$fname.html"
  #
  echo "<p>Saved to $fname.html"
fi

echo "$img"

check_newton

display_prog 

echo "</form></body></html>"
