#! /bin/dash
#
# update project file
#
# this script is invoked via chroot+setuidgid
#
# cwd is $top/$sub
#
# R. Perry, Aug. 2003
#
# ZZ marks local customization sections

PATH="/usr/bin"; export PATH

RMCMTS="/vecr/bin/rmcmts"
INDENT="/usr/bin/indent"

mode="$1"
user="$2"
sub="$3"
file="$4"
args="$5"

case "$color" in
   "blue") color="green";;
 "green") color="blue";;
       *) color="blue";;
esac

###
# function to check for test option/script
#
check_tester() {
  tester=""
  #
  case "$file" in
   p1.c|p1.h|p1.cc|p1.hh|p1.java|p1.sh|scan.l|parse.y) tester="test1" ;;
   p2.c|p2.h|p2.cc|p2.hh|p2.java|p2.sh) tester="test2" ;;
   p3.c|p3.h|p3.cc|p3.hh|p3.java|p3.sh) tester="test3" ;;
   p4.c|p4.h|p4.cc|p4.hh|p4.java|p4.sh) tester="test4" ;;
  esac
  #
  tester_link=""
  #
  if [ -n "$tester" -a -x "$tester" ]; then
   tester_link="- <a href=\"/prog/${COURSE}?tester+${sub}+${file}\"
    target=\"bot\" title=\"run test on $file\">test</a>"
  fi
}

###
# function to display subdir, home menu
#
menu1() {
  echo "<a href=\"/prog/${COURSE}?edit+${sub}\" target=\"_top\"
    title=\"go to ${sub} directory\">${sub}</a>
  - <a href=\"/prog/${COURSE}\" target=\"_top\"
    title=\"go to Home directory\">Home</a>
  $tester_link
  - <a href=\"/prog/${COURSE}?preproc+${sub}+${file}\"
    title=\"run the preprocessor\" target=\"bot\">preproc</a>
  - <a href=\"/prog/${COURSE}?asm+${sub}+${file}\"
    title=\"generate assembly code\" target=\"bot\">asm</a>
  - <a href=\"/prog/${COURSE}?asmopt+${sub}+${file}\"
    title=\"generate optimized assembly code\" target=\"bot\">asmopt</a>
  - <a href=\"/prog/${COURSE}?disasm+${sub}+${file}\"
    title=\"disassemble\" target=\"bot\">disasm</a>
  - <a href=\"/prog/${COURSE}?cerr+${sub}+${file}\"
    title=\"insert compile error messages as comments\" target=\"bot\">cerr</a>
  - <a href=\"/prog/${COURSE}?clean+${sub}+${file}\"
    title=\"make clean\" target=\"bot\">clean</a>
  - <a href=\"/prog/${COURSE}?edit+${sub}+${file}\"
    title=\"reload ${file}\">reload</a>
  - <a href=\"/prog/${COURSE}?rmcmts+${sub}+${file}\"
    title=\"remove comments\">rmcmts</a>
  - <a href=\"/prog/${COURSE}?indent+${sub}+${file}\"
    title=\"run indent code formatter\">indent</a>
  - <a href=\"/prog/${COURSE}?frameset+${sub}+${file}\"
    title=\"turn on frames\" target=\"_top\">+frames</a>
  - <a href=\"/prog/${COURSE}?edit+${sub}+${file}\"
    title=\"turn off frames\" target=\"_top\">-frames</a>
<br>"

}

###
# function to display compile/run menu
#
menu2() {
  echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\" target=\"bot\" spellcheck=\"false\">
  <a href=\"/prog/${COURSE}?compile+${sub}+${file}\" target=\"bot\"
   title=\"run the compiler\">Compile</a>
  <input type=submit name=run value=\"Run\" title=\"run the program\">
  <input type=submit name=crun value=\"C/Run\"
   title=\"compile and run the program\">
  <input type=submit name=debug value=\"Debug\" title=\"run using the debugger\">
   <input name=wav type=image src=\"/vecr/images/sound.gif\"
    title=\"play output as WAV audio\" border=0 align=absmiddle>
  args: <input name=args size=25 maxlength=80 value=\"$args\"
   title=\"command-line arguments, or input, or command\">
  <input type=submit name=input value=\"Input\"
   title=\"use args as standard input\">
  <input type=submit name=cmd value=\"Command\"
   title=\"execute args as a command\">
  </form>"
}

###
# function to display Update menu
#
menu3() {
  echo "<input type=submit name=update value=\"Update\" title=\"save changes\">
  <font color=$color>&lt;-save changes</font>
  <input name=plot type=image title=\"2D plot\""
  # ZZ
  # if [ "$COURSE" = "osp" -a "$sub" = "a7" ]; then
  #   echo "src=\"/vecr/images/maze.gif\" width=32 height=32"
  # else
  #   echo "src=\"/vecr/images/Stats.gif\""
  # fi
  echo "src=\"/vecr/images/Stats.gif\"
    border=0 align=absmiddle>
   <input name=splot type=image src=\"/vecr/images/Chart3.gif\"
    title=\"3D plot\" border=0 align=absmiddle>
   <input name=image type=image src=\"/vecr/images/image.png\"
    title=\"view PPM image as PNG\" border=0 align=absmiddle>
   args: <input name=args size=25 maxlength=80 value=\"$args\"
    title=\"command-line arguments\">"
}

#   <input type=submit name=IO value=\"IO\" title=\"run using interactive I/O\">
#   <input type=submit name=IOD value=\"IO/D\" title=\"run debugger interactively\">

case "$mode" in

  update)
	#
	# make backup, copy file
	#
	rm -f "$file.OLD"
	mv "$file" "$file.OLD"
	case "$file" in
	  Makefile) tr -d '\r' | sed -e 's/^ /	/' -e '/^$/d' > "$file" ;;
	  *)	    tr -d '\r' | sed               -e '/^$/d' > "$file" ;;
	esac
	;;

  upload)
	#
	# make backup, upload file
	#
	read boundary
	while read header
	do
	  if [ -z "$header" ]; then
	    break
	  fi
	done
	tmpfile="upload.$$"
	rm -f "$tmpfile"
	sed -e '/^$/d' -e "/^$boundary/,\$d" > "$tmpfile"
	#
	if [ -s "$tmpfile" ]; then
	  rm -f "$file.OLD"
	  mv "$file" "$file.OLD"
	  mv "$tmpfile" "$file"
	else
	  rm -f "$tmpfile"
	fi
	;;

  copy)
	#
	# make backup, copy file
	#
	read fname rest
	fpath="/vecr/copy/$fname"
	#
	if [ -n "$fname" -a -r "$fpath" ]; then
	  rm -f "$file.OLD"
	  mv "$file" "$file.OLD"
	  cp "$fpath" "$file"
	fi
	exit 0
	;;

  edit|rmcmts|indent)
	#
	# create links as needed, display edit page
	#
	for p in ../../../proto/${sub}/*
	do
	  f=`basename "$p"`
	  if [ ! -r "$f" ]; then
	    ln -s "$p" "$f"
	  fi
	done
	#
	# dynamically create additional files here...
	#
	if [ ! -r "$file" ]; then
	  touch "$file"
	fi
#--	#
#--	if [ ! -r "$file" ]; then
#--	 case "$COURSE" in # ZZ
#--	 #
#--	  osp)
#--	   case "$sub" in
#--	    a5) case "$file" in
#--		 p1.c) /vecr/osp/a5/make.sh "$user" > p1.c ;;
#--		    *) touch "$file" ;;
#--		esac
#--		;;
#--	    *) touch "$file"
#--		;;
#--	   esac
#--	   ;;
#--	 #
#--	  *) touch "$file"
#--	   ;;
#--	 #
#--	 esac
#--	fi
	#
	echo "Content-type: text/html\n"
	echo "<html><head><title>${user} - ${COURSE} - ${sub}/${file}</title>
	  </head><body bgcolor=white text=black>
	  <h2>${user} - ${COURSE} - ${sub}/${file}"
	 #
	  if [ "$sub" = "HTML" ]; then
	    case "$file" in
	      *.sh|*.c|*.h|*.cc|*.hh|*.java) echo " (run output Content-Type will be HTML)"
 	      ;;
	    esac
	  # ZZ:
	  #-- elif [ "$COURSE/$sub/$file" = "osp/a9/scan.l" ]; then
	  #--   echo " (run output Content-Type will be HTML)"
	  fi
	 #
	 case "$file" in
	   Makefile) echo " (leading blank will be converted to tab)" ;;
	 esac
	#
	echo "</h2>"
	#
	# ZZ
	#
	case "${COURSE}/${sub}/${file}" in
	  osp/a1/p1.txt|osp/a1/p1.c|osp/a1/p2.c)
	    echo "<font color=red>*** ${sub}/${file} was collected Sun Aug 30 11:44:16 EDT 2020 ***</font><p>"
	  ;;
	  osp/a1/p3.c|osp/a1/p4.c)
	    echo "<font color=red>*** ${sub}/${file} was collected Sun Sep  6 08:06:19 EDT 2020 ***</font><p>"
	  ;;
	esac
	#
	check_tester
	menu1
	menu2
	echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\" spellcheck=\"false\">"
	menu3
	#
	# echo "<br><textarea name=data rows=60 cols=75>" #  wrap=\"off\"
	#
	# the line numbers div idea is from Ryan Collins
	#
	# added font-family:monospace 2015-08-26, suggested by Brian Kim
	#
	echo "<div style=\"position:relative;font-size:14;overflow-y:hidden;font-family:monospace;\">
	<div style=\"position:absolute;top:4px;left:0px;font-size:14;height:100%;line-height:16px;font-family:monospace;\">"
	#
	echo "1<br>2<br>3<br>4<br>5<br>6<br>7<br>8<br>9<br>10<br>
	11<br>12<br>13<br>14<br>15<br>16<br>17<br>18<br>19<br>20<br>
	21<br>22<br>23<br>24<br>25<br>26<br>27<br>28<br>29<br>30<br>
	31<br>32<br>33<br>34<br>35<br>36<br>37<br>38<br>39<br>40<br>
	41<br>42<br>43<br>44<br>45<br>46<br>47<br>48<br>49<br>50<br>
	51<br>52<br>53<br>54<br>55<br>56<br>57<br>58<br>59<br>60</div>"
	#
	echo "<textarea name=\"data\" rows=\"60\" cols=\"90\" style=\"position:relative;left:18px;top:0px;font-size:14;line-height:16px;font-family:monospace;\">"
	#
	case "$mode" in
	  edit)
	    sed -e 's/\&/\&amp;/g' -e 's/</\&lt;/g' -e 's/>/\&gt;/g' "$file"
	    ;;
	  rmcmts) # remove comments
	    $RMCMTS < "$file" | sed -e '/^  *$/d' -e 's/\&/\&amp;/g' -e 's/</\&lt;/g' -e 's/>/\&gt;/g'
	    ;;
	  indent) # indent
	    $INDENT -bli0 -c0 -dj -npcs -nsaf -nsai -nsaw < "$file" |
	      sed -e '/^  *$/d' -e 's/\&/\&amp;/g' -e 's/</\&lt;/g' -e 's/>/\&gt;/g'
	    ;;
	esac
	# echo "</textarea><br>"
	echo "</textarea></div><br>"
	echo "<input type=submit name=update value=\"Update\"
	 title=\"save changes\"><font color=$color>&lt;-save changes</font>
	 <input type=\"hidden\" name=\"color\" value=\"$color\"></form>"
	echo "<br><form method=POST enctype=\"multipart/form-data\"
	action=\"/prog/${COURSE}?${sub}+${file}\">
	<input type=file name=upload size=30 maxlength=80
	 title=\"browse local files for upload\">
	<input type=submit name=upload value=\"Upload\"
	 title=\"upload file\"></form>"
	#
	case "$COURSE" in # ZZ
	  osp)
	    case "$file" in
	      *.c)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/C/C.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.txt)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/txt/txt.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.ss)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/ss/ss.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
#	      *.sh)
#		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
#		awk '{ print "<option>"$0; }' < /vecr/copy/sh/sh.list
#		echo "</select><input type=submit name=copy value=\"Copy\"
#		 title=\"copy selected file\"></form>"
#		;;
	      *.l)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/flex/flex.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.y)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/bison/bison.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	    esac
	    ;;
	  fc)
	    case "$file" in
	      *.c)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/Etter4e/C.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.txt)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/Etter4e/data.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.ss)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/ss/ss.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	    esac
	    ;;
	  cpp)
	    case "$file" in
	      *.cc)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/cpp/cpp.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.txt)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/cpp/txt.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.ss)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/ss/ss.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.l)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/flex/flex.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	      *.y)
		echo "<form method=POST action=\"/prog/${COURSE}?${sub}+${file}\"><select name=data>"
		awk '{ print "<option>"$0; }' < /vecr/copy/bison/bison.list
		echo "</select><input type=submit name=copy value=\"Copy\"
		 title=\"copy selected file\"></form>"
		;;
	    esac
	    ;;
	esac
	#
	echo "</body></html>"
	;;

  *)
	echo "Content-type: text/plain\n"
	echo "Bad mode"
	exit 1
	;;
esac
